#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os, gtk
import gtk
from classes.project import project
from classes import messagebox, info
from windows.SimpleGtkBuilderApp import SimpleGtkBuilderApp

# init the foreign language
from language import Language_Init


class frmAbout(SimpleGtkBuilderApp):

	def __init__(self, path="About.ui", root="aboutdialog1", domain="OpenShot", version="0.0.0", project=None, **kwargs):
		SimpleGtkBuilderApp.__init__(self,  os.path.join(project.UI_DIR, path), root, domain, **kwargs)

		# Add language support
		_ = Language_Init.Translator(project).lang.gettext
		
		# set version from constructor
		self.aboutdialog1.set_version(version)
		
		# init authors
		authors = []
		for person in info.CREDITS['code']:
			name = person['name']
			email = person['email']
			authors.append("%s <%s>" % (name, email))
		self.aboutdialog1.set_authors(authors)
		
		# init documenters
		authors = []
		for person in info.CREDITS['documentation']:
			name = person['name']
			email = person['email']
			authors.append("%s <%s>" % (name, email))
		self.aboutdialog1.set_documenters(authors)
		
		# init translators
		self.aboutdialog1.set_translator_credits("Translation credits are located on LaunchPad:\nhttps://translations.launchpad.net/openshot")



	def new(self):
		print "A new %s has been created" % self.__class__.__name__


	def on_aboutdialog1_close(self, widget, *args):
		print "on_aboutdialog1_close called with self.%s" % widget.get_name()

		# close the window
		self.frmAbout.destroy()

	def on_aboutdialog1_response(self, widget, *args):
		print "on_aboutdialog1_close called with self.%s" % widget.get_name()

		# close the window
		self.aboutdialog1.destroy()

def main():
	aboutdialog1 = frmAbout()
	aboutdialog1.run()

if __name__ == "__main__":
	main()



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os, sys, threading, time
from classes import files, profiles

try:
	import mlt
except ImportError:
	print "*** ERROR: MLT Python bindings failed to import ***"
	

class thumbnailer ( threading.Thread ):
	""" This class is designed to always be running during OpenShot.  It's a seperate thread that 
	is always waiting to inspect video and audio files, generate thumbnails, etc... """

	def set_project(self, project):
		 """ Associate the OpenShot project file with this threaded class. """
		 self.project = project
		 
	def GetFile(self, file_name, all_frames=False):
		 """ Use this method to generate an OpenShotFile object based on the URL (or file location)
		 of a video or audio file. Each time you call this method, it will lock this thread (and OpenShot's
		 main thread) until it has finished. """

		 try:
			 
			 # determine name and location of thumbnail image
			 self.file_name = file_name
			 self.thumbnail_path = ""
			 self.file_type = "video"
			 self.label = ""
			 project_path = self.project.folder
			 myPath = self.file_name
			 (dirName, fileName) = os.path.split(myPath)
			 (fileBaseName, fileExtension)=os.path.splitext(fileName)
			 fileExtension = fileExtension.replace(".", "")
			 actual_thumbnail_path = project_path + "/thumbnail/" + fileBaseName + "_" + fileExtension + "_1.png"

			 if all_frames == False:
			 	# just get 1 thumbnail frame
			 	self.thumbnail_path = project_path + "/thumbnail/" + fileBaseName + "_" + fileExtension + "_%d.png"
			 	
			 	# set the profile
			 	self.profile = mlt.Profile("quarter_ntsc")

			 else:
			 	# export all frames of the video
			 	self.thumbnail_path = dirName + "/" + fileBaseName + "/" + fileBaseName + "_%d.png"
			 	
			 	# set the profile
			 	self.profile = profiles.mlt_profiles(self.project).get_profile(self.project.project_type)

			 # re-init the mlt factory
			 mlt.Factory.init()
			 	
			 # Create the producer
			 self.p = mlt.Producer( self.profile, '%s' % self.file_name )
			 
			 # Check if clip is valid (otherwise a seg fault)
			 if self.p.is_valid() == False:
			 	return None
			 
		 	 # create the consumer
		 	 self.c = mlt.Consumer(self.profile, "avformat", self.thumbnail_path)

			 # set some consumer properties
			 self.c.set("real_time", 0)
			 self.c.set("vcodec", "png")

			 # determine length of clip in seconds
			 max_frames = float(self.p.get_length()) - 1.0
			 producer_fps = float(self.p.get_fps())
			 calculate_length = max_frames / producer_fps

			 # determine if this is an image
			 is_image = False
			 if self.p.get_length() == 15000:
			 	# images always have exactly 15000 frames
			 	is_image = True
			 	self.file_type = "image"
			 	
			 	# set the max length of the image to 300 seconds (i.e. 5 minutes)
			 	max_frames = producer_fps * 300
			 	
			 	# set the length to 300 seconds (i.e. 5 minutes)
			 	calculate_length = float(300)

			 height = 0
			 width = 0
			 if self.p.get("height"):
				  height = int(self.p.get("height"))
			 if self.p.get("width"):
				  width = int(self.p.get("width"))
		 

			 # set thumbnail image (if no height & width are detected)
			 if (height == False or width == False) and (is_image == False):
				  self.thumbnail_path = ""
				  self.file_type = "audio"
				  
			 # set size of images
			 if is_image:
			 	height = 1
			 	width = 1

			 # get the 1st frame (if not exporting all frames)
			 if all_frames == False:
			 	self.p = self.p.cut(1, 1)
			 else:
			 	# mark as image seq
			 	self.label = "Image Sequence"
			 	self.file_type = "image sequence"
			 
			 # connect the producer and consumer
			 self.c.connect( self.p )

			 # Start the consumer, and lock the thread until it's done (to prevent crazy seg fault errors)
			 # Only start if the media item has a thumbnail location (i.e. no audio thumbnails)
			 if self.thumbnail_path:
			 	self.c.run()

	
			 # create an openshot file object
			 newFile = files.OpenShotFile(self.project)
			 newFile.name = self.file_name
			 newFile.length = calculate_length
			 newFile.thumb_location = actual_thumbnail_path
			 newFile.videorate = (self.p.get_fps(), 0)
			 newFile.height = height
			 newFile.width = width
			 newFile.max_frames = max_frames
			 newFile.fps = producer_fps
			 newFile.file_type = self.file_type
			 newFile.label = self.label

			 # return the OpenShotFile object
			 return newFile
		
		 except Exception:
			print "Failed to import file: %s" % file_name



	def run ( self ):
		 """ This is the main method on this thread.  This method should not return anything, or the 
		 thread will no longer be active... and thus will no longer be able to inspect media files. """
		 
		 self.amAlive = True
		 self.file_name = ""
		 self.c = None
		 self.p = None

		 # init the factory, and load a small video size / profile
		 mlt.Factory().init()
		 self.profile = mlt.Profile("quarter_ntsc")

		 # this loop will continue as long as OpenShot is running
		 while self.amAlive:
			  time.sleep( 1 ) 

		 # clear all the MLT objects
		 self.p = None
		 self.c = None
		 self.profile = None
		 self.f = None
		 
	def get_thumb_at_frame(self, filename, frame=1, new_name=""):
		'''by passing a value in new_name, this will extract a 
		   full size frame (based on the project profile).'''
		   
		self.file_name = filename
		
		project_path = self.project.folder
		myPath = self.file_name
		(dirName, fileName) = os.path.split(myPath)
		(fileBaseName, fileExtension)=os.path.splitext(fileName)
		fileExtension = fileExtension.replace(".", "")
		
		mlt.Factory.init()
		
				
		# set the profile
		if new_name == "":
			self.profile = mlt.Profile("quarter_ntsc")
		else:
			self.profile = profiles.mlt_profiles(self.project).get_profile(self.project.project_type)
		
		# Create the producer
		self.p = mlt.Producer( self.profile, '%s' % self.file_name )
				 
		# Check if clip is valid (otherwise a seg fault)
		if self.p.is_valid() == False:
			return None
		
		
		if new_name == "":
			# just get 1 thumbnail frame
			self.thumbnail_path = project_path + "/thumbnail/" + fileBaseName + "_" + fileExtension + "_%d.png"
		else:
			#for snapshots, use the new file name
			#don't use the thumbnail path for the new file
			self.thumbnail_path = project_path + "/" + new_name
		
		# create the consumer
		self.c = mlt.Consumer(self.profile, "avformat", self.thumbnail_path)
	
		# set some consumer properties
		self.c.set("real_time", 0)
		self.c.set("vcodec", "png")
		
		#get the frame
		self.p = self.p.cut(frame, frame)
		
		# connect the producer and consumer
		self.c.connect( self.p )
	
		# Only start if the media item has a thumbnail location (i.e. no audio thumbnails)
		if self.thumbnail_path:
			self.c.run()
			
		 



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.


import os
import gtk
import xml.dom.minidom as xml

from classes import profiles, project, messagebox, tree
from windows.SimpleGtkBuilderApp import SimpleGtkBuilderApp
from xdg.IconTheme import *

# init the foriegn language
import language.Language_Init as Language_Init

class PreferencesMgr(SimpleGtkBuilderApp):
	
	
	def __init__(self, path="Preferences.ui", root="frmPreferences", domain="OpenShot", form=None, project=None, **kwargs):
		SimpleGtkBuilderApp.__init__(self, os.path.join(project.UI_DIR, path), root, domain, **kwargs)
		
		# Add language support
		_ = Language_Init.Translator(project).lang.gettext
		self._ = _
		
		self.project = project
		self.form = form

		if getIconPath("openshot"):
			self.frmPreferences.set_icon_from_file(getIconPath("openshot"))
		
		#populate the profiles dropdown
		self.populate_profiles()

		#populate the themes
		for dir in os.listdir(self.project.THEMES_DIR):
			self.cmbThemes.append_text(dir)
		
		# populate output mode combo
		for output_mode in ["sdl", "sdl_preview"]:
			self.cmbOutputModes.append_text(output_mode)
			
			
		#populate the codecs & formats
		self.VCodecList = gtk.ListStore(str)
		self.tvVCodecs.set_model(self.VCodecList)
		tree.treeviewAddGeneralTextColumn(self.tvVCodecs,_('Video Codecs'),0)
		
		self.ACodecList = gtk.ListStore(str)
		self.tvACodecs.set_model(self.ACodecList)
		tree.treeviewAddGeneralTextColumn(self.tvACodecs,_('Audio Codecs'),0)
		
		self.FormatsList = gtk.ListStore(str)
		self.tvFormats.set_model(self.FormatsList)
		tree.treeviewAddGeneralTextColumn(self.tvFormats,_('Formats'),0)	
		self.populate_codecs()
		
			
		#populate form objects
		self.valImageLength.set_value(float(self.form.settings.general["imported_image_length"]))
		self.valHistoryStackSize.set_value(float(self.form.settings.general["max_history_size"]))
		self.txtMeltCommandName.set_text(self.form.settings.general["melt_command"])
		self.txtBlenderCommand.set_text(self.form.settings.general["blender_command"])
		theme_name = self.form.settings.general["default_theme"]
		self.set_dropdown_values(theme_name, self.cmbThemes)
		self.set_dropdown_values(self.form.settings.general["output_mode"], self.cmbOutputModes)

		
		#show the form
		self.frmPreferences.show_all()
		
	def set_default_profile(self, profile_name):
		# set the selected profile
		self.set_dropdown_values(profile_name, self.cmbProfiles)
		
	def populate_profiles(self):

		# init the list of possible project types / profiles
		self.profile_list = profiles.mlt_profiles(self.project).get_profile_list()
		
		# loop through each profile, and add it to the dropdown
		self.liststore2.clear()
		#self.cmbProfiles.clear()
		for file_name, p in self.profile_list:
			# append profile to list
			self.cmbProfiles.append_text(str(file_name))
			
		# re-select default profile
		self.set_default_profile(self.form.settings.general["default_profile"])
		if not self.cmbProfiles.get_active_text():
			self.set_default_profile("DV NTSC")
			
		
	def populate_codecs(self):
		
		#populate the codecs
		
		#video codecs		
		for codec in self.form.vcodecs:
			self.VCodecList.append([codec])
		
		#audio codecs
		for acodec in self.form.acodecs:
			self.ACodecList.append([acodec])
		
		#formats
		for format in self.form.vformats:
			self.FormatsList.append([format])
			
	
	def on_btnReload_clicked(self, widget, *args):
		
		#clear the codecs from the form object
		#and repopulate the listviews
		
		self.VCodecList.clear()
		self.ACodecList.clear()
		self.FormatsList.clear()
		
		self.form.vcodecs[:] = []
		self.form.acodecs[:] = []
		self.form.vformats[:] = []
		
		melt_command = self.form.settings.general["melt_command"]
		self.form.get_avformats(melt_command)
		
		self.populate_codecs()
		
		
	def on_btnClose_clicked(self, widget, *args):
		#write the values from the form to the dictionary objects
		self.form.settings.general["imported_image_length"] = self.valImageLength.get_text()
		self.form.settings.general["default_theme"] = self.cmbThemes.get_active_text()
		self.form.settings.general["default_profile"] = self.cmbProfiles.get_active_text()
		self.form.settings.general["melt_command"] = self.txtMeltCommandName.get_text()
		self.form.settings.general["blender_command"] = self.txtBlenderCommand.get_text()
		self.form.settings.general["output_mode"] = self.cmbOutputModes.get_active_text()

		# save settings
		self.form.settings.save_settings_to_xml()
		
		# close the window
		self.frmPreferences.destroy()
		
	def on_valImageLength_value_changed(self, widget, *args):
		self.form.settings.general["imported_image_length"] = self.valImageLength.get_value_as_int()
		
	def on_cmbProfiles_changed(self, widget, *args):
		self.form.settings.general["default_profile"] = self.cmbProfiles.get_active_text()
		
	def on_valHistoryStackSize_value_changed(self, widget, *args):
		self.form.settings.general["max_history_size"] = self.valHistoryStackSize.get_value_as_int()
		
	def on_txtMeltCommandName_focus_out_event(self, widget, *args):
		self.form.settings.general["melt_command"] = self.txtMeltCommandName.get_text()
		
	def on_txtBlenderCommand_focus_out_event(self, widget, *args):
		self.form.settings.general["blender_command"] = self.txtBlenderCommand.get_text()
		
	def on_cmbOutputModes_changed(self, widget, *args):
		self.form.settings.general["output_mode"] = self.cmbOutputModes.get_active_text()

	def on_cmbThemes_changed(self, widget, *args):
		
		self.form.settings.general["default_theme"] = self.cmbThemes.get_active_text()
		
		#reload the theme example image
		theme_name = self.cmbThemes.get_active_text()
		
		# update theme on main form
		self.form.project.theme = theme_name
		self.form.update_icon_theme()
		self.form.refresh()


	def on_btnManageProfiles_clicked(self, widget, *args):
		print "on_btnManageProfiles_clicked"
		from windows import Profiles
		Profiles.frmProfilesManager(form=self.form, parent=self, project=self.project)
			
	
	def set_dropdown_values(self, value_to_set, combobox):
		
		model = combobox.get_model()
		iter = model.get_iter_first()
		while True:
			# get the value of each item in the dropdown
			value = model.get_value(iter, 0)
			
			# check for the matching value
			if value_to_set == value:			
				
				# set the item as active
				combobox.set_active_iter(iter)
				break
		
			# get the next item in the list
			iter = model.iter_next(iter)
			
			# break loop when no more dropdown items are found
			if iter is None:
				break
			
	
				

class Settings:
	#set some initial values.
	#The first time Openshot is run, the settings file won't exist,
	#so use these default values.
	#When the settings are loaded from the config file, these default
	#values will be overwritten with the file values.
	general = {
		"imported_image_length" : "7",
		"default_theme" : "blue_glass",
		"default_profile" : "DV NTSC",
		"project_file_type" : "ascii",
		"max_history_size" : "20",
		"melt_command" : "melt",
		"blender_command" : "blender",
		"output_mode" : "sdl",
		}
	
	app_state = {
		"window_height" : "710",
		"window_width" : "900",
		"window_maximized" : "False",
		"import_folder" : "None",
		"toolbar_visible" : "True",
		"history_visible" : "False",
		"vpane_position" : "370",
		"hpane_position" : "450",
		"clip_property_window_width" : "775",
		"clip_property_window_height" : "345",
		"clip_property_window_maximized" : "False",
		"clip_property_hpane_position" : "260",
		
		}
	
	sections = {
		"general" : general,
		"app_state" : app_state
		
		}
	
	def __init__(self, project):
		"""Constructor"""
		
		# Add language support
		translator = Language_Init.Translator(project)
		_ = translator.lang.gettext
		
		self.project = project
	
	def load_settings_from_xml(self):
		settings_path = os.path.join(self.project.USER_DIR, "config.xml")
		
		#Load the settings from the config file, if it exists
		if os.path.exists(settings_path):
			
			try:
				xmldoc = xml.parse(settings_path)
			except xml.xml.parsers.expat.ExpatError:
				# Invalid or empty config file
				self.save_settings_to_xml()
				return
			#loop through each settings section and load the values
			#into the relevant dictionary
			for section, section_dict in self.sections.iteritems():
				for key, value in section_dict.iteritems():
					try:
						element = xmldoc.getElementsByTagName(key)
						section_dict[key] = element[0].childNodes[0].data
						
						# be sure theme exists
						if key == "default_theme":
							if os.path.exists(os.path.join(self.project.THEMES_DIR, section_dict[key])) == False:
								# DOES NOT EXIST, change to default
								section_dict[key] = "blue_glass"
						
					except IndexError:
						#the list index will go out of range if there is
						#an extra item in the dictionary which is
						#not in the config file.
						pass
		
		else:
			# no config file found, create one
			self.save_settings_to_xml()
			
	def save_settings_to_xml(self):
		settings_path = os.path.join(self.project.USER_DIR, "config.xml")
		
		#update each xml element with the current dictionary values
		if os.path.exists(settings_path):
			try:
				xmldoc = xml.parse(settings_path)
			except xml.xml.parsers.expat.ExpatError:
				# Invalid or empty config file, create new blank dom
				messagebox.show(_("OpenShot Warning"), _("Invalid or empty preferences file found, loaded default values"))
				xmldoc = xml.Document()
				root_node = xmldoc.createElement("settings")
				xmldoc.appendChild(root_node)
				
				# create a node for each section
				for section, section_dict in self.sections.iteritems():
					section_node = xmldoc.createElement(section)
					root_node.appendChild(section_node)
		else:
			# missing config file, create new blank dom
			xmldoc = xml.Document()
			root_node = xmldoc.createElement("settings")
			xmldoc.appendChild(root_node)
			
			# create a node for each section
			for section, section_dict in self.sections.iteritems():
				section_node = xmldoc.createElement(section)
				root_node.appendChild(section_node)
				
		
		for section, section_dict in self.sections.iteritems():
			for key, value in section_dict.iteritems():
				try:
					element = xmldoc.getElementsByTagName(key)
					if element:
						element[0].childNodes[0].data = section_dict[key]
					else:
						#there is no matching element in the xml, 
						#we need to add one
						new_element = xmldoc.createElement(key)
						parent = xmldoc.getElementsByTagName(section)
						parent[0].appendChild(new_element)
						txt = xmldoc.createTextNode(str(value))
						new_element.appendChild(txt)
						
				except IndexError:
					pass
	
		# save settings
		self.write_to_settings_file(xmldoc)
			
			
			
			
					
	def write_to_settings_file(self, xmldoc):
		#write the updated xml document to the config file
		filename = os.path.join(self.project.USER_DIR, "config.xml")
		
		try:
			file = open(filename, "wb") 
			file.write(xmldoc.toxml("UTF-8"))
			#xmldoc.writexml(file, indent='', addindent='    ', newl='', encoding='UTF-8')
			file.close()
		except IOError, inst:
			messagebox.show(_("OpenShot Error"), _("Unexpected Error '%s' while writing to '%s'." % (inst, filename)))
		
		
def main():
	frm_prefs = PreferencesMgr()
	frm_titles.run()

if __name__ == "__main__":
	main()



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os, uuid, locale 

########################################################################
class keyframe:
	"""This class represents a media clip on the timeline."""

	#----------------------------------------------------------------------
	def __init__(self, frame, height, width, x, y, alpha):
		"""Constructor"""
		
		# init variables for keyframe object
		self.frame = frame
		self.height = height
		self.width = width
		self.x = x
		self.y = y
		self.alpha = alpha
		self.unique_id = str(uuid.uuid1())	
		
	def set_all(self, height, width, x, y, alpha):
		""" Set all properties with 1 method. """
		if height != None:
			self.height = height
		if width != None:	
			self.width = width
		if x != None:
			self.x = x
		if y != None:
			self.y = y
		if alpha != None:
			self.alpha = alpha
		
	def generate_string(self):
		""" Generate the MLT keyframe string for the XML: 0=0%,0%:100%x100%:100; -1=0%,0%:100%x100%:100 """
		
		# generate string
		output = "%d=%s%%,%s%%:%s%%x%s%%:%s; " % (self.frame, locale.str(self.x), locale.str(self.y), locale.str(self.width), locale.str(self.height), locale.str(self.alpha * 100.0))
		
		# return string
		return output
		
		
		
		
		
		
		
		
		
		
		
		



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os, sys, time 
import threading
import gobject
from classes import profiles

# init the foreign language
from language import Language_Init

class player ( threading.Thread ):
	
	def __init__(self, project, main_form, file_name, mode=None):
		self.project = project
		self.main_form = main_form
		self.file_name = file_name
		
		# Add language support
		_ = Language_Init.Translator(project).lang.gettext
		self._ = _
		
		# set the FPS
		self.fps = self.project.fps()
		
		# update mode
		if mode:
			# set the mode
			self.mode = mode

		# call base class
		threading.Thread.__init__(self)


	def set_project(self, project, main_form, file_name, mode=None, render_options = None, override_path = None):
		self.project = project
		self.main_form = main_form
		self.file_name = file_name
		self.render_options = render_options
		
		# set the FPS
		self.fps = self.project.fps()
		
		# update mode
		if mode:
			# set the mode
			self.mode = mode
			
		# override mode
		if override_path:
			self.override_path = override_path
		else:
			self.override_path = None
			
		
	def set_profile(self, profile_name, load_xml = True):

		# set the MLT profile object... specifiec in the project properties
		self.consumer_stop()
		self.profile = profiles.mlt_profiles(self.project).get_profile(profile_name)

		# set the FPS
		self.fps = self.project.fps()
		
		# re-load the mlt objects (i.e. the producer & consumer)
		if load_xml:
			self.load_xml()

		
	def load_xml(self):
		
		# Import required modules
		import time
		import sys
		import gobject
		
		# get reference to translate gettext method
		_ = self._
		
		try:
			import mlt
		except ImportError:
			print "*** ERROR: MLT Python bindings failed to import ***"

		# re-init the mlt factory
		mlt.Factory.init()

		# Create the consumer
		if self.mode == "render":
			# RENDER MOVIE mode
			
			# stop the consumer
			self.consumer_stop()
	
			# Create the producer
			self.p = mlt.Producer( self.profile, 'xml:%s' % self.file_name)

			# get export file path
			folder = self.render_options["folder"]
			file = self.render_options["file"]
			format = self.render_options["f"]
			export_path = "%s.%s" % (os.path.join(folder, file), format)
			
			# create consumer
			self.c = mlt.Consumer( self.profile, "avformat", export_path)
			
			# set some RENDER specific options
			self.c.set("real_time", -1)
			
			# set render options
			if self.render_options["export_to"] == _("Image Sequence"):
				# image seq
				self.c.set("vcodec" , self.render_options["vcodec"])
			else:
				# video & audio
				self.c.set("f" , format)
				self.c.set("vcodec" , self.render_options["vcodec"])
				self.c.set("b" , self.render_options["b"])
				self.c.set("acodec" , self.render_options["acodec"])
				self.c.set("ar" , self.render_options["ar"])
				self.c.set("ac" , self.render_options["ac"])
				self.c.set("ab" , self.render_options["ab"])
			
				if self.render_options["vcodec"] == "libx264":
					self.c.set("minrate", "0")
					self.c.set("b_strategy", "1")
					self.c.set("subcmp", "2")
					self.c.set("cmp", "2")
					self.c.set("coder", "1")
					self.c.set("flags", "+loop")
					self.c.set("flags2", "dct8x8")
					self.c.set("qmax", "51")
					self.c.set("subq", "7")
					self.c.set("qmin", "10")
					self.c.set("qcomp", "0.6")
					self.c.set("qdiff", "4")
					self.c.set("trellis", "1")
				if format == "dvd":
					# stolen from ffmpeg.c, void opt_target(const char *arg)
					self.c.set("maxrate", "9000000")
					self.c.set("minrate", "0")
					self.c.set("bufsize", "1835008")
					self.c.set("packetsize", "2048")
					self.c.set("muxrate", "10080000")

		else:
			# stop the consumer (if needed)
			if str(self.main_form.settings.general["output_mode"]) == "sdl_preview":
				# stop the consumer to prevent a crash with sdl_preview consumer
				self.consumer_stop()
			
			# Create the producer
			self.p = mlt.Producer( self.profile, 'xml:%s' % self.file_name)

			# refresh sdl and pause video
			self.pause()

		# connect the producer and consumer
		self.c.connect( self.p )

		# start consumer
		self.c.start()


	def run ( self ):
		# FIXME: Is this relevant - doesn't seem to be used in this method?
		self.override_path = None
		self.alternate_progress_bar = None

		# Import required modules
		import time
		import sys
		import gobject
		
		try:
			import mlt
		except ImportError:
			print "*** ERROR: MLT Python bindings failed to import ***"

		# track wheather the thread is playing or not
		self.isPlaying = False 
		
		# track if this thread should die
		self.amAlive = True
		
		# Start the mlt system
		self.f = mlt.Factory().init( )
		
		# set the MLT profile object... specifiec in the project properties
		self.profile = profiles.mlt_profiles(self.project).get_profile(self.project.project_type)
		
		# Create the producer
		self.p = mlt.Producer( self.profile, 'xml:%s' % self.file_name)
		
		if self.p:
			# set speed to zero (i.e. pause)
			self.pause()
			
			# PREVIEW mode
			self.c = mlt.Consumer( self.profile, str(self.main_form.settings.general["output_mode"]) )
			self.c.set("real_time", 1)
		
			# Connect the producer to the consumer
			self.c.connect( self.p )
			
			# Start the consumer
			self.c.start()
			
			# Get the FPS
			self.fps = self.project.fps()
			
			# init the render percentage
			self.fraction_complete = 0.0		
			
			# Wait until the user stops the consumer
			while self.amAlive:
				
				# get current frame
				current_frame = self.p.position()

				# only calculate position / percentage when playing
				if self.c.is_stopped() == False:

					# move play head
					new_time = float(self.p.position()) / float(self.fps)
					
					if self.mode == "render":
						# update Export Dialog Progress Bar
						self.fraction_complete = (float(self.p.position()) / float(self.p.get_length() - 1))
						if self.project.form.frmExportVideo:
							gobject.idle_add(self.project.form.frmExportVideo.update_progress, self.fraction_complete)
						
					elif self.mode == "preview":
						
						if self.alternate_progress_bar:
							# update alternateive progress bar (if any) of video
							# this is used by the clip properties window 
							percentage_complete = (float(self.p.position()) / float(self.p.get_length() - 1)) * float(100)
							if self.alternate_progress_bar:
								gobject.idle_add(self.alternate_progress_bar.set_value, percentage_complete)
							
						else:
							# update play-head
							if self.project.sequences[0]:
								gobject.idle_add(self.project.sequences[0].move_play_head, new_time)
			
							# update progress bar of video 
							percentage_complete = (float(self.p.position()) / float(self.p.get_length() - 1)) * float(100)
							if self.main_form.hsVideoProgress:
								gobject.idle_add(self.main_form.hsVideoProgress.set_value, percentage_complete)
								gobject.idle_add(self.main_form.scroll_to_playhead)
								
							# pause video when 100%
							if percentage_complete == 100:
								self.pause()
						
						
					elif self.mode == "override":
						# update progress bar of video 
						percentage_complete = (float(self.p.position()) / float(self.p.get_length() - 1)) * float(100)
						if self.main_form.hsVideoProgress:
							gobject.idle_add(self.main_form.hsVideoProgress.set_value, percentage_complete)

					# wait 1/5 of a second
					time.sleep( 0.2 )

					
				else:
					if self.mode == "render":
						# update Export Dialog Progress Bar
						if self.fraction_complete > 0.0:
							# update progress bar to 100%
							if self.project.form.frmExportVideo:
								gobject.idle_add(self.project.form.frmExportVideo.update_progress, 1.0)
							
							# reset the fraction
							self.fraction_complete = 0.0

					# wait 1 second
					time.sleep( 1 )



			# clear all the MLT objects
			self.consumer_stop()
			self.p = None
			self.c = None
			self.profile = None
			self.f = None
			
		else:
			# Diagnostics
			print "ERROR WITH %s" % self.file_name

	def set_progress_bar(self, pbar):
		""" Set the progress bar that this thread should update """
		self.alternate_progress_bar = pbar

	def clear_progress_bar(self):
		""" Set the progress bar that this thread should update """
		self.alternate_progress_bar = None
		
	def get_progress_bar(self):
		""" Get the progress bar object """
		return self.alternate_progress_bar
	
	def close_window(self, window):
		""" Let this thread close a window """
		window.destroy()

	def move_play_head(self, new_time):
		# call this thread move the play_head
		gobject.idle_add(self.project.sequences[0].move_play_head, new_time)
		

	def pause(self):
		# pause the video
		self.isPlaying = False
		self.p.set_speed(0)
		
			
	def play(self):
		# play the video
		self.isPlaying = True
		self.p.set_speed(1)


	def seek(self, frame_number):

		# pause frame (if the video is playing)
		if self.isPlaying:	
			# pause
			self.pause()
			
			# wait for 2/10ths of a second (for mlt to catch up to 
			# the correct seek position
			#time.sleep(0.2)
			
		# seek to a specific frame 
		self.p.seek(frame_number) 
		
		# refresh sdl
		self.refresh_sdl()
		
		
	def consumer_stop(self):
		""" Stop the MLT consumer """
		
		# stop consumer
		self.c.stop()
		
		
	def new_sdl_consumer(self):
		""" Create a new SDL consumer and attach it to the producer """
		
		try:
			import mlt
		except ImportError:
			print "*** ERROR: MLT Python bindings failed to import ***"
		
		# create and set new consumer
		self.c = mlt.Consumer( self.profile, str(self.main_form.settings.general["output_mode"]) )
		self.c.set("real_time", 1)
		
		# pause the consumer
		self.pause()
		
		
	def position(self):
		""" Get the MLT position (current frame number) """
		
		# return the current frame
		return self.p.position()
	
	
	def get_speed(self):
		""" Get the MLT speed """
		
		# return the MLT speed
		return self.p.get_speed()
	
	def set_speed(self, new_speed):
		""" Set the MLT speed """
		
		# set the speed of the producer
		self.p.set_speed(new_speed)
		
	
	def get_length(self):
		""" Get the length of the MLT producer """
		
		# return the length of the XML file, in frames
		return self.p.get_length()
	
		
		
	def refresh_sdl(self):
		""" Tell MLT to refresh the SDL image. """ 
		
		# update the refresh property on the consumer
		self.c.set("refresh", 1)



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.



#   ----------------------------- EXAMPLE CODE ----------------------------
#   Use this snippet on any module that needs to support translations.  Once this 
#   snippet is in place, you can use the _("") method to set all your strings.
#
#	   import language.Language_Init as Language_Init
#
#	   # Add this to each method that needs to translate strings
#	   _ = Language_Init.Translator().lang.gettext
#
#	   # To mark a string as 'translatable', use this syntax:
#	   MyString = _("Here is my string")



# ----------------------- GENERATE THE POT FILE --------------------
# To generate the POT file, which contains all translatable text in
# OpenShot, run the Generate_POT.py script (in this same folder)
# Example:  $ python Generate_POT.py
#
# Ignore warnings, and the new POT file should be located at
# /openshot/openshot/locale/OpenShot/OpenShot.pot



# Command to Launch Program in any language
# LANG=es python OpenShot.py
#
# *** TROUBLESHOOTING TIPS:  If you get the following error: 
# (process:27990): Gtk-WARNING **: Locale not supported by C library. Using the fallback 'C' locale.
# 
# SOLUTION:  Step 1) Modify this file and add your language: /var/lib/locales/supported.d/local
#            Step 2) sudo dpkg-reconfigure locales
#            Step 3) Be sure the correct translation file localted at /usr/local/share/openshot/openshot/language/
#               NOTE:  fr <> fr_FR.  You must match the language code exactly.
# -------------------------------------------------------------------------


# import modules needed for language translation & gtk support
import os, sys
import gettext, locale
import gtk


########################################################################
class Translator:
	"""Class that contains functions to initiate language translations"""  
	
	#----------------------------------------------------------------------
	def __init__(self, project):
		"""Constructor"""  
		self.project = project
		
		# Setup foreign language support
		langs = []
		lc, encoding = locale.getdefaultlocale()

		if (lc):
			langs = [lc]			
		language = os.environ.get('LANGUAGE', None)
		if (language):
			langs += language.split(":")
		# Add languages... for testing
		#langs += ["es", "fr"]
		locale.setlocale(locale.LC_ALL)
		locale.setlocale(locale.LC_NUMERIC, 'POSIX')
		locale.bindtextdomain("OpenShot", self.project.LOCALE_DIR)
		
		gettext.textdomain("OpenShot")
		gettext.bindtextdomain("OpenShot", self.project.LOCALE_DIR)
		
		gettext.install(domain="OpenShot")

		# This reference is used by other classes to define the _ method 
		self.lang = gettext.translation("OpenShot", self.project.LOCALE_DIR, languages = langs, fallback = True)